######################################################################################
#  Script name: Optimal pick-a-winner same cutoff finder                             #
#                                                                                    #
#  Purpose: Through grid search, find the same optimal cutoff for pick-a-winner      #
#  comparison in both efficay and toxicity endpoints under the H1 and H4. Record     #
#  the OC summary table for different choices of cutoff in the searching process.    #
#                                                                                    #
#  Authors: Yanzhao Wang, Ziji Yu, Jianchang Lin                                     #
#                                                                                    #
#  Date: 08/18/2022                                                                  #
#                                                                                    #
#  Email address: yzwang1994@gmail.com                                               #
#                                                                                    #
#  Comments: This simulation code generates data under assumptions from H1 to H8 in  #
#  the manuscript. The selection of theta2 only depends on the H1 and H4. In the grid#
#  search, only the OC from H1 to H4 are recorded for different theta2. Once the     #
#  optimal cutoff is selected, its OC under H1 and H8 can be calculated at the end   #
#                                                                                    #
######################################################################################

######################################################################################
#  User's input                                                                      #
## H_0: ORR = .3, TOX = 0.3; H_a: ORR = 0.45, TOX =0.15; eta_ORR = 0.15,eta_TOX = 0.1
## H_0: ORR = .3, TOX = 0.75; H_a: ORR = 0.45, TOX =0.6; eta_ORR = 0.15,eta_TOX = 0.15
## External parameters
N = 30 # maximum sample size
L =2 # The number of stages
n = N/L # The number of new patients enrolled at each stage
p_ORR_0 = 0.3 # meaningful efficacy under H0
p_TOX_0 = 0.75 # tolerable toxicity under H0
p_ORR_1 = 0.45 # meaningful efficacy under Ha
p_TOX_1 = 0.60 # tolerable toxicity under Ha
n.sim =10000 # number of simulation
phi = 1## Strength of association between ORR and TOX
eta_ORR = 0.15 ## Superiority margin of ORR
eta_TOX = 0.15 ## Superiority margin of TOX
## Selection error
alpha_L = 0.3 ## Not Recommend Low  when Low is better
alpha_H = 0.3## Not Recommend High when High is better
######### Pick up BOPII stopping Boundary (Grid search tunning parameters)
grids.num_lmd = 1000; # grid resolution
grids.num_nu=1000;# grid resolution
lmd_L = 0.1; # Lower bound for lambda
lmd_H = 0.9; # Upper bound for lambda
nu_L = 0.1; # Lower bound for nu
nu_H=2; # Upper bound for nu
# To get the marginal of ORR and TOX in a multinomial vector
collapse_ORR = c(1,1,0,0) ## collapse pi1, pi2
collapse_TOX = c(1,0,1,0) ## collapse pi1, pi3
alpha =0.05; # Type I error control;
# From the website https://www.trialdesign.org/one-page-shell.html#BOP2

## H_0: ORR = .3, TOX = 0.3; H_a: ORR = 0.45, TOX =0.15; eta_ORR = 0.15,eta_TOX = 0.1
# stopBoundary_mat_opt = cbind(c(15,30),c(4,11),c(5,8)) ## From Web for 30, L =2

# stopBoundary_mat_opt = cbind(c(25,50,75),c(6,14,25),c(8,15,21)) ## From Web for 75

## H_0: ORR = .3, TOX = 0.75; H_a: ORR = 0.45, TOX =0.6; eta_ORR = 0.15,eta_TOX = 0.15
# stopBoundary_mat_opt0 = cbind(c(15,30),c(4,11),c(11,22)) ## From Web for 30, L =2


######################################################################################


cat("The final outputs will be two OC tables of BOPII and of DODII\nUnder H1 ~ H8 and optimal theta2\n
    they are stats_tbl_BOPII_only_tbl,theta2_tbl_2nd,theta2_opt")
library(parallel)
library(doParallel)
library(dplyr)
library(ggplot2)
num_core = detectCores()-1
source("user_defined.R")


# The ORR and TOX assumptions for those 8 scenarios 
## H_1
p_ORR_High_H1 = p_ORR_1+eta_ORR;p_TOX_High_H1 = p_TOX_1
p_ORR_Low_H1 = p_ORR_1;p_TOX_Low_H1 = p_TOX_1

## H_2
p_ORR_High_H2 = p_ORR_1+eta_ORR;p_TOX_High_H2 = p_TOX_1
p_ORR_Low_H2 = p_ORR_1;p_TOX_Low_H2 = p_TOX_1-eta_TOX

## H_3
p_ORR_High_H3 = p_ORR_1;p_TOX_High_H3 = p_TOX_1
p_ORR_Low_H3 = p_ORR_1;p_TOX_Low_H3 = p_TOX_1

## H_4
p_ORR_High_H4 = p_ORR_1;p_TOX_High_H4 = p_TOX_1
p_ORR_Low_H4 = p_ORR_1;p_TOX_Low_H4 = p_TOX_1-eta_TOX


## H_5
p_ORR_High_H5 = p_ORR_0;p_TOX_High_H5 = p_TOX_0
p_ORR_Low_H5 = p_ORR_0;p_TOX_Low_H5 = p_TOX_0

## H_6
p_ORR_High_H6 = p_ORR_0;p_TOX_High_H6 = p_TOX_0
p_ORR_Low_H6 = p_ORR_0;p_TOX_Low_H6 = p_TOX_1

## H_7
p_ORR_High_H7 = p_ORR_1;p_TOX_High_H7 = p_TOX_0
p_ORR_Low_H7 = p_ORR_0;p_TOX_Low_H7 = p_TOX_0

## H_8
p_ORR_High_H8 = p_ORR_1;p_TOX_High_H8 = p_TOX_0
p_ORR_Low_H8 = p_ORR_0;p_TOX_Low_H8 = p_TOX_1






set.seed(1024)
###########################################################################
cat("We are generating generate stopping boundary for BOPII design.\n")
## Construct the grid
lmd_vec = seq(lmd_L,lmd_H,length.out=grids.num_lmd)
nu_vec = seq(nu_L,nu_H,length.out = grids.num_nu)

## Simulation for L stages
# set.seed(1024)
pi_with_phi_H0 = gen_pis(p_ORR_0,p_TOX_0,phi=1)
pi_with_phi_H1 = gen_pis(p_ORR_1,p_TOX_1,phi=1)
sim_array_H0 = Trial_sim(pi_with_phi_H0,n.sim=n.sim,L,n.interim=n)
sim_array_H1 = Trial_sim(pi_with_phi_H1,n.sim=n.sim,L,n.interim=n)
cl=makeCluster(num_core)
registerDoParallel(cl)

lambda_nu_opt = as.matrix(expand.grid(lmd_vec,nu_vec))
## Stopping boundaries for 
lambda_nu_opt_list=split(lambda_nu_opt,row(lambda_nu_opt))
stopBoundary_mat_list= parLapply(cl,lambda_nu_opt_list,
                                 stopBoundary_cal,L=L,n=n, p_ORR_0=p_ORR_0,p_TOX_0=p_TOX_0,
                                 collapse_ORR=collapse_ORR,collapse_TOX=collapse_TOX,
                                 pi_with_phi = pi_with_phi_H0,n0=1)

stopBoundary_mat_list_unique_raw = unique(stopBoundary_mat_list)
stopBoundary_mat_list_unique = lapply(stopBoundary_mat_list_unique_raw,function(x){
  if(!anyNA(x)){
    return(x)
  }
})
stopBoundary_mat_list_unique = stopBoundary_mat_list_unique[lapply(stopBoundary_mat_list_unique,length)>0]


t1e_vec = foreach(list_i=1:length(stopBoundary_mat_list_unique),.combine = c) %dopar%
  mean(apply(sim_array_H0,MARGIN = 3,power_cal,stopBoundary_mat=stopBoundary_mat_list_unique[[list_i]]))
power_vec = foreach(list_i=1:length(stopBoundary_mat_list_unique),.combine = c) %dopar%
  mean(apply(sim_array_H1,MARGIN = 3,power_cal,stopBoundary_mat=stopBoundary_mat_list_unique[[list_i]]))
param.opt = which(power_vec==max(power_vec[t1e_vec<=alpha]))
stopBoundary_mat_opt = stopBoundary_mat_list_unique[[param.opt]]
#############################################################################


### Simulation for Comparison
for(case_i in 1:8){
  p_ORR_High_temp = get(paste0('p_ORR_High_H',case_i))
  p_TOX_High_temp = get(paste0('p_TOX_High_H',case_i))
  p_ORR_Low_temp = get(paste0('p_ORR_Low_H',case_i))
  p_TOX_Low_temp = get(paste0('p_TOX_Low_H',case_i))
  pi_with_phi_High_temp = gen_pis(p_ORR_High_temp,p_TOX_High_temp,phi=1)
  pi_with_phi_Low_temp = gen_pis(p_ORR_Low_temp,p_TOX_Low_temp,phi=1)
  sim_array_Low_temp = Trial_sim(pi_with_phi_Low_temp,n.sim=n.sim,L,n.interim=n)
  sim_array_High_temp = Trial_sim(pi_with_phi_High_temp,n.sim=n.sim,L,n.interim=n)
  sim_array_High_Low_temp = array(NA,dim = c(4,L,n.sim,2))
  sim_array_High_Low_temp[,,,1] = sim_array_High_temp
  sim_array_High_Low_temp[,,,2] = sim_array_Low_temp
  assign(paste0("sim_array_High_Low_Compare_H",case_i),sim_array_High_Low_temp)
  assign(paste0('pi_with_phi_High_H',case_i),pi_with_phi_High_temp)
  assign(paste0('pi_with_phi_Low_H',case_i),pi_with_phi_Low_temp)
}

## List of 8 scienarios along with their prior assumptions
sim_array_High_Low_Compare_list = list(sim_array_High_Low_Compare_H1,
                                       sim_array_High_Low_Compare_H2,
                                       sim_array_High_Low_Compare_H3,
                                       sim_array_High_Low_Compare_H4,
                                       sim_array_High_Low_Compare_H5,
                                       sim_array_High_Low_Compare_H6,
                                       sim_array_High_Low_Compare_H7,
                                       sim_array_High_Low_Compare_H8)

pi_with_phi_Low_list = list(pi_with_phi_Low_H1,
                            pi_with_phi_Low_H2,
                            pi_with_phi_Low_H3,
                            pi_with_phi_Low_H4,
                            pi_with_phi_Low_H5,
                            pi_with_phi_Low_H6,
                            pi_with_phi_Low_H7,
                            pi_with_phi_Low_H8)

pi_with_phi_High_list = list(pi_with_phi_High_H1,
                             pi_with_phi_High_H2,
                             pi_with_phi_High_H3,
                             pi_with_phi_High_H4,
                             pi_with_phi_High_H5,
                             pi_with_phi_High_H6,
                             pi_with_phi_High_H7,
                             pi_with_phi_High_H8)







## Calculating the OC for different theta2.
theta2_vec = seq(0.51,0.99,by=0.01)
system.time({
  theta2_grid_raw=foreach(theta2 = theta2_vec,.combine = rbind) %:%
    foreach(case_id = 1:4, .combine = rbind) %dopar%
    stopBoundary_cal_compare(theta2_ORR=rep(theta2,L),theta2_TOX = rep(theta2,L),case_id=case_id,
                                       sim_array_High_Low_Compare_list=sim_array_High_Low_Compare_list,
                                       pi_with_phi_High_list=pi_with_phi_High_list,
                                       pi_with_phi_Low_list=pi_with_phi_Low_list,stopBoundary_mat_opt = stopBoundary_mat_opt,n=n)
})


# Tabulating
theta2_tbl=tibble(
  case = theta2_grid_raw[,10],
  theta2 = as.numeric(theta2_grid_raw[,8]),
  Recommend_High_only = as.numeric(theta2_grid_raw[,1]),
  Recommend_Low_only = as.numeric(theta2_grid_raw[,2]),
  Recommend_Both = as.numeric(theta2_grid_raw[,3]),
  Recommend_None = as.numeric(theta2_grid_raw[,4]),
  n_sample_size = as.numeric(theta2_grid_raw[,5]),
  n_sample_size_High_dose = as.numeric(theta2_grid_raw[,6]),
  n_sample_size_Low_dose = as.numeric(theta2_grid_raw[,7])
)


#### SaveRDS
# saveRDS(theta2_grid_raw,file="Saved RDS/ORR3045TOX3015SameCutoff_n75_L3_raw_details.rds")
# saveRDS(theta2_tbl,file="Saved RDS/ORR3075TOX3060SameCutoff_n30_L2.rds")
#### ReadRDS
# theta2_tbl = readRDS("C:\\Users\\ILE1552\\Takeda\\2022 summer intern working folder - Documents\\program\\Saved RDS\\(FOR MANUSCRIPT)Same theta2 for Eff and Tox\\ORR3075TOX3060SameCutoff_n30_L2.rds")




#### Optimization criteria to select the optimal theta2
theta2_tbl_alpha_L_controlled=
  theta2_tbl%>%filter(case==4&Recommend_High_only+Recommend_None<=alpha_L)%>%
               select(theta2)

theta2_tbl_alpha_H_controlled=theta2_tbl%>%filter(case==1&Recommend_Low_only+Recommend_None<=alpha_H)%>%
  select(theta2)


theta2_cands_tbl= theta2_tbl_alpha_H_controlled%>%inner_join(theta2_tbl_alpha_L_controlled,
                                                             by=c("theta2"))%>%
  distinct()


theta2_powers_tbl = theta2_cands_tbl%>%left_join(theta2_tbl,by=c("theta2"))%>%
  filter(case%in%c(1,4))


beta_H_tibble = theta2_powers_tbl%>%
  filter(case==1)%>%
  select(theta2,Recommend_High_only)

beta_L_tibble = theta2_powers_tbl%>%
  filter(case==4)%>%
  select(theta2,Recommend_Low_only)


beta_H_tibble$Recommend_Low_only = beta_L_tibble$Recommend_Low_only

beta_HL_tibble = beta_H_tibble%>%
  group_by(theta2)%>%
  mutate(min_power = min(Recommend_High_only,Recommend_Low_only))


theta2_opt = beta_HL_tibble%>%
  filter(min_power==max(beta_HL_tibble$min_power))%>%
  select(theta2)%>%tail(1)%>%pull()

## BOPII only performance
stats_tbl_BOPII_only_tbl = tibble()
for(case_id in 1:8){
  sim_array_High_Low_Compare_case = sim_array_High_Low_Compare_list[[case_id]]
  pi_with_phi_High_case=pi_with_phi_High_list[[case_id]]
  pi_with_phi_Low_case = pi_with_phi_Low_list[[case_id]]
  
  system.time({
    Stats_vec_BOPII_only=foreach(i=1:n.sim,.combine = rbind) %dopar%
      get_stats_vec_BOPII_ONLY(i,
                               case_id=case_id,sim_array_High_Low_Compare_list=sim_array_High_Low_Compare_list,
                               pi_with_phi_High_list=pi_with_phi_High_list,
                               pi_with_phi_Low_list=pi_with_phi_Low_list,
                               stopBoundary_mat_opt=stopBoundary_mat_opt,
                               n=n)
  })
  
  stats_tbl_BOPII_only_temp=tibble(
    case = case_id,
    Recommend_High_only = as.numeric(Stats_vec_BOPII_only[,1]),
    Recommend_Low_only = as.numeric(Stats_vec_BOPII_only[,2]),
    Recommend_Both = as.numeric(Stats_vec_BOPII_only[,3]),
    Recommend_None = as.numeric(Stats_vec_BOPII_only[,4]),
    n_sample_size = as.numeric(Stats_vec_BOPII_only[,5]),
    n_sample_size_High_dose = as.numeric(Stats_vec_BOPII_only[,6]),
    n_sample_size_Low_dose = as.numeric(Stats_vec_BOPII_only[,7])
  )
  stats_tbl_BOPII_only_temp_summary=stats_tbl_BOPII_only_temp%>%
    group_by(case)%>%
    summarise_all(mean)
  
  stats_tbl_BOPII_only_tbl=rbind(stats_tbl_BOPII_only_tbl,stats_tbl_BOPII_only_temp_summary)
}




stats_tbl_BOPII_only_tbl%>%
  mutate_at(vars(n_sample_size,n_sample_size_High_dose,n_sample_size_Low_dose),funs(round(
    .,2)))%>%
  mutate_at(vars(Recommend_High_only,Recommend_Low_only,
                 Recommend_Both,Recommend_None),funs(round(
                   .*100,2)))%>%
  View()



### When the optimal theta2 is selected, plug in to calculate the OC for all 8 scenarios.
theta2_vec = theta2_opt
system.time({
  theta2_grid_raw=foreach(theta2 = theta2_vec,.combine = rbind) %:%
    foreach(case_id = 1:8, .combine = rbind) %dopar%
    stopBoundary_cal_compare(theta2_ORR=rep(theta2,L),theta2_TOX = rep(theta2,L),case_id=case_id,
                             sim_array_High_Low_Compare_list=sim_array_High_Low_Compare_list,
                             pi_with_phi_High_list=pi_with_phi_High_list,
                             pi_with_phi_Low_list=pi_with_phi_Low_list,stopBoundary_mat_opt = stopBoundary_mat_opt,n=n)
})

stopCluster(cl)
theta2_tbl_2nd=tibble(
  case = theta2_grid_raw[,10],
  theta2 = as.numeric(theta2_grid_raw[,8]),
  Recommend_High_only = as.numeric(theta2_grid_raw[,1]),
  Recommend_Low_only = as.numeric(theta2_grid_raw[,2]),
  Recommend_Both = as.numeric(theta2_grid_raw[,3]),
  Recommend_None = as.numeric(theta2_grid_raw[,4]),
  n_sample_size = as.numeric(theta2_grid_raw[,5]),
  n_sample_size_High_dose = as.numeric(theta2_grid_raw[,6]),
  n_sample_size_Low_dose = as.numeric(theta2_grid_raw[,7])
)

theta2_tbl_2nd%>%
  filter(theta2 == theta2_opt)%>%
  mutate_at(vars(n_sample_size,n_sample_size_High_dose,n_sample_size_Low_dose),funs(round(
    .,2)))%>%
  mutate_at(vars(Recommend_High_only,Recommend_Low_only,
                 Recommend_Both,Recommend_None),funs(round(
                   .*100,2)))%>%
  View()

print(paste("The optimal theta2 is ",theta2_opt))

