######################################################################################
#  Script name: Plotting integer decision rule                            
#                                                                                    #
#  Purpose: Convert the probability cutoff to integer decision rule    #
#                                                                                    #
#  Authors: Yanzhao Wang, Ziji Yu, Jianchang Lin                                     #
#                                                                                    #
#  Date: 08/18/2022                                                                  #
#                                                                                    #
#  Email address: yzwang1994@gmail.com                                               #
#                                                                                    #
#  Comments: User's need to provide the optimal cutoffs for the endpoints, each cutoff
#  is in the form of the vector of length L (interim number). 
#                                                                                    #
######################################################################################


######################################################################################
#  User's input                                                                      #
## H_0: ORR = .3, TOX = 0.3; H_a: ORR = 0.45, TOX =0.15; eta_ORR = 0.15,eta_TOX = 0.1
## H_0: ORR = .3, TOX = 0.75; H_a: ORR = 0.45, TOX =0.6; eta_ORR = 0.15,eta_TOX = 0.15
## External parameters

N = 30 # maximum sample size
L =2 # The number of stages

### probability cutoffs (calibrated in the simulation section)
theta2_ORR = rep(0.62,L); theta2_TOX = rep(0.62,L)

n = N/L # The number of new patients enrolled at each stage
p_ORR_0 = 0.3 # meaningful efficacy under H0
p_TOX_0 = 0.75 # tolerable toxicity under H0
p_ORR_1 = 0.45 # meaningful efficacy under Ha
p_TOX_1 = 0.60 # tolerable toxicity under Ha

phi = 1## Strength of association between ORR and TOX
eta_ORR = 0.15 ## Superiority margin of ORR
eta_TOX = 0.15 ## Superiority margin of TOX


######### Pick up BOPII stopping Boundary
## Need this to restrict the searching area
# From the website https://www.trialdesign.org/one-page-shell.html#BOP2

## H_0: ORR = .3, TOX = 0.3; H_a: ORR = 0.45, TOX =0.15; eta_ORR = 0.15,eta_TOX = 0.1
# stopBoundary_mat_opt = cbind(c(10,20),c(4,11),c(5,8)) ## From Web for 30, L =2

# stopBoundary_mat_opt = cbind(c(25,50,75),c(6,14,25),c(8,15,21)) ## From Web for 75

## H_0: ORR = .3, TOX = 0.75; H_a: ORR = 0.45, TOX =0.6; eta_ORR = 0.15,eta_TOX = 0.15
stopBoundary_mat_opt = cbind(c(10,20),c(4,11),c(11,22)) ## From Web for 30, L =2


library(dplyr)
library(ggplot2)
library(ggpubr)
library(parallel)
library(doParallel)
source("C:\\Users\\ILE1552\\Takeda\\2022 summer intern working folder - Documents\\program\\DODII code\\user_defined.R")
num_core = detectCores()-1


## H_1
p_ORR_High_H1 = p_ORR_1+eta_ORR;p_TOX_High_H1 = p_TOX_1
p_ORR_Low_H1 = p_ORR_1;p_TOX_Low_H1 = p_TOX_1

## H_2
p_ORR_High_H2 = p_ORR_1+eta_ORR;p_TOX_High_H2 = p_TOX_1
p_ORR_Low_H2 = p_ORR_1;p_TOX_Low_H2 = p_TOX_1-eta_TOX

## H_3
p_ORR_High_H3 = p_ORR_1;p_TOX_High_H3 = p_TOX_1
p_ORR_Low_H3 = p_ORR_1;p_TOX_Low_H3 = p_TOX_1

## H_4
p_ORR_High_H4 = p_ORR_1;p_TOX_High_H4 = p_TOX_1
p_ORR_Low_H4 = p_ORR_1;p_TOX_Low_H4 = p_TOX_1-eta_TOX



collapse_ORR = c(1,1,0,0) ## collapse pi1, pi2
collapse_TOX = c(1,0,1,0) ## collapse pi1, pi3

for(case_i in 1:4){
  p_ORR_High_temp = get(paste0('p_ORR_High_H',case_i))
  p_TOX_High_temp = get(paste0('p_TOX_High_H',case_i))
  p_ORR_Low_temp = get(paste0('p_ORR_Low_H',case_i))
  p_TOX_Low_temp = get(paste0('p_TOX_Low_H',case_i))
  pi_with_phi_High_temp = gen_pis(p_ORR_High_temp,p_TOX_High_temp,phi=1)
  pi_with_phi_Low_temp = gen_pis(p_ORR_Low_temp,p_TOX_Low_temp,phi=1)
  assign(paste0('pi_with_phi_High_H',case_i),pi_with_phi_High_temp)
  assign(paste0('pi_with_phi_Low_H',case_i),pi_with_phi_Low_temp)
}




cl = makeCluster(num_core)
registerDoParallel(cl)


### Get integer boundary
ORR_boundary_interim_tbl_list = list()
TOX_boundary_interim_tbl_list = list()
for(L_i in 1:L){
  
  # pos_prob_compare_ORR_temp is P(ORR_h - ORR_l > eta_ORR | X)  
  pos_prob_ORR_combinations = foreach(X_H_i = (stopBoundary_mat_opt[L_i,2]+1):(L_i*n),.combine = cbind) %:%
    foreach(X_L_i=(stopBoundary_mat_opt[L_i,2]+1):(L_i*n),.combine=rbind) %dopar%
    compare_pos_cal(eta = eta_ORR,x_1 = c(X_H_i,0,0,L_i*n-X_H_i),x_2 = c(X_L_i,0,0,L_i*n-X_L_i),
                    collapse_type = collapse_ORR,pi_with_phi_1 = pi_with_phi_High_H3,
                    pi_with_phi_2 = pi_with_phi_Low_H3,n0 = 1)
  
  
  # pos_prob_compare_TOX_temp is P(TOX_h - TOX_l < eta_TOX | X)  
  pos_prob_TOX_combinations = 1-foreach(X_H_i = 0:(stopBoundary_mat_opt[L_i,3]-1),.combine = cbind) %:%
    foreach(X_L_i = 0:(stopBoundary_mat_opt[L_i,3]-1),.combine=rbind) %dopar%
    compare_pos_cal(eta = eta_TOX,x_1 = c(X_H_i,0,0,L_i*n-X_H_i),x_2 = c(X_L_i,0,0,L_i*n-X_L_i),
                    collapse_type = collapse_TOX,pi_with_phi_1 = pi_with_phi_High_H3,
                    pi_with_phi_2 = pi_with_phi_Low_H3,n0 = 1)
  
  
  
  
  pos_prob_ORR_combinations_tbl = tibble(
    num_High_ORR = rep((stopBoundary_mat_opt[L_i,2]+1):(L_i*n),each = length((stopBoundary_mat_opt[L_i,2]+1):(L_i*n))),
    num_LOW_ORR = rep((stopBoundary_mat_opt[L_i,2]+1):(L_i*n),length((stopBoundary_mat_opt[L_i,2]+1):(L_i*n))),
    ORR_decision = c(ifelse(pos_prob_ORR_combinations<1-theta2_ORR[L_i],"ORR_high_sig_not_better",
                            ifelse(pos_prob_ORR_combinations>theta2_ORR[L_i],"ORR_high_sig_better","Unclear")))
  )
  
  
  
  
  pic_ORR_temp = pos_prob_ORR_combinations_tbl%>%
    ggplot()+
    geom_rect(aes(xmin=num_LOW_ORR,
                  xmax=num_LOW_ORR+.75,
                  ymin=num_High_ORR,
                  ymax = num_High_ORR+.75,fill = ORR_decision))+
    scale_fill_manual(values =c("ORR_high_sig_better"="green",
                                "Unclear"="yellow",
                                "ORR_high_sig_not_better"="red"),
                      labels=c("High is better","Unclear","High is not better"),name="Decision for ORR")+
    ylab("Responses in High dose")+xlab("Responses in Low dose")+
    ggtitle(label = paste0("CutoffORR=",round(theta2_ORR[L_i],2)," CutoffTOX=",round(theta2_TOX[L_i],2)))
  
  
  
  pos_prob_TOX_combinations_tbl = tibble(
    num_High_TOX = rep(0:(stopBoundary_mat_opt[L_i,3]-1),each = length(0:(stopBoundary_mat_opt[L_i,3]-1))),
    num_LOW_TOX = rep(0:(stopBoundary_mat_opt[L_i,3]-1),length(0:(stopBoundary_mat_opt[L_i,3]-1))),
    TOX_decision = c(ifelse(pos_prob_TOX_combinations>theta2_TOX[L_i],"TOX_high_sig_not_worse",
                            ifelse(pos_prob_TOX_combinations<1-theta2_TOX[L_i],"TOX_high_sig_worse","Unclear")))
  )
  
  pic_TOX_temp =pos_prob_TOX_combinations_tbl%>%
    ggplot()+
    geom_rect(aes(xmin=num_LOW_TOX,
                  xmax=num_LOW_TOX+.75,
                  ymin=num_High_TOX,
                  ymax = num_High_TOX+.75,fill = TOX_decision))+
    scale_fill_manual(values =c("TOX_high_sig_not_worse"="green",
                                "Unclear"="yellow",
                                "TOX_high_sig_worse"="red"),
                      labels=c("High is not worse","Unclear","High is worse"),name="Decision for TOX")+
    ylab("Toxicities in High dose")+xlab("Toxicities in Low dose")+
    ggtitle(label = paste0("CutoffORR=",round(theta2_ORR[L_i],2)," CutoffTOX=",round(theta2_TOX[L_i],2)))
  filenames_ORR_temp = paste0("ORR_stopping_boundary_for_interim",L_i,"_same_theta_func_sample.pdf")
  filenames_TOX_temp = paste0("TOX_stopping_boundary_for_interim",L_i,"_same_theta_func_sample.pdf")
  ggsave(filename = filenames_ORR_temp,plot = pic_ORR_temp,width = 5,height = 5)
  ggsave(filename = filenames_TOX_temp,plot = pic_TOX_temp,width = 5,height = 5)
  ORR_boundary_interim_tbl_list[[L_i]] = pos_prob_ORR_combinations_tbl
  TOX_boundary_interim_tbl_list[[L_i]] = pos_prob_TOX_combinations_tbl
}

stopCluster(cl)
pic_TOX_temp
pic_ORR_temp

  

